bold = "bold"
italic = "italic"
accessory = "#4A7D95"
badge = "#B00007"
or_badge = "#0DCAF0"


# dictionary-entry
# --- entry-header-content --- #
eh_h1_word = "#2E8B57"
eh_fg_entry_num = "#0A1B27"
eh_bg_entry_num = "#F4F4F4"
eh_entry_num = "{} on {}".format(eh_fg_entry_num, eh_bg_entry_num)
eh_word_type = "#FD7E14"
eh_word_syllables = "#757575"
eh_pron = "#4A7D95" # UNUSED

# --- verb tenses, noun plurals, adjective comparative forms --- #
wt = "#525A5B" # UNUSED

# --- other word forms --- #
wf = "#4A7D95"
wf_type = "#4A7D95"

# --- transitive / intransitive --- #
tran = "yellow" # UNUSED

# --- one meaning of one entry --- #
meaning_num = "#757575"
meaning_letter = "#757575"
meaning_link = "#4A7D95"
meaning_content = "#4A7D95"
meaning_badge = badge
meaning_arrow = "#8B5742"
meaning_keyword = "#0DCAF0"
meaning_sentence = "#757575"

# --- dxnls --- #
dxnls_title = "#2E8B57"
dxnls_content = "#757575"


# phrases
ph_title = "#2E8B57"
ph_item = "#4A7D95"


# synonyms
syn_title = "#0F3850"
syn_item = "#757575"
syn_em = "#2E8B57"
syn_label = "#0F3850"


# examples on the web
eg_title = "#0F3850"
eg_sentence = "#757575"
eg_word = "#2E8B57"


# related phrases
rph_title = "#0F3850"
rph_em = "#2E8B57"
rph_item = "#4A7D95"


# nearby entries
nearby_title = "#0F3850"
nearby_em = "#2E8B57"
nearby_word = "#757575"
nearby_item = "#4A7D95"


# dict name
dict_name = "#757575"


# Word of the Day
wod_title = "#2E8B57"
wod_type = eh_word_type
wod_syllables = eh_word_syllables
wod_subtitle = "#0F3850"
wod_sent = meaning_sentence
